@@ -7,7 +7,7 @@
  *
  * The GnuTLS is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
+ * as published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
@@ -124,12 +124,12 @@ _gnutls_srp_send_params (gnutls_session_t session,
   if (session->security_parameters.entity == GNUTLS_CLIENT)
     {
       gnutls_srp_client_credentials_t cred = (gnutls_srp_client_credentials_t)
-        _gnutls_get_cred (session->key, GNUTLS_CRD_SRP, NULL);
+        _gnutls_get_cred (session, GNUTLS_CRD_SRP, NULL);
 
       if (cred == NULL)
         return 0;
 
-      priv = gnutls_malloc (sizeof (*priv));
+      priv = gnutls_calloc (1, sizeof (*priv));
       if (priv == NULL)
         return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
 
