@@ -311,6 +311,55 @@ cleanup:
   return ret;
 }
 
+/**
+ * gnutls_pkcs11_privkey_status:
+ * @key: Holds the key
+ *
+ * Checks the status of the private key token.
+ *
+ * Returns: this function will return non-zero if the token 
+ * holding the private key is still available (inserted), and zero otherwise.
+ * 
+ * Since: 3.1.9
+ *
+ **/
+int
+gnutls_pkcs11_privkey_status (gnutls_pkcs11_privkey_t key)
+{
+  ck_rv_t rv;
+  int ret;
+  struct pkcs11_session_info _sinfo;
+  struct pkcs11_session_info *sinfo;
+  ck_object_handle_t obj;
+  struct ck_session_info session_info;
+
+  if (key->sinfo.init != 0)
+    {
+      sinfo = &key->sinfo;
+      obj = key->obj;
+    }
+  else
+    {
+      sinfo = &_sinfo;
+      memset(sinfo, 0, sizeof(*sinfo));
+      FIND_OBJECT (sinfo, &key->pin, obj, key);
+    }
+  
+  rv = (sinfo->module)->C_GetSessionInfo (sinfo->pks, &session_info);
+  if (rv != CKR_OK)
+    {
+      ret = 0;
+      goto cleanup;
+    }
+  ret = 1;
+
+cleanup:
+  if (sinfo != &key->sinfo)
+    pkcs11_close_session (sinfo);
+
+  return ret;
+}
+
 /**
  * gnutls_pkcs11_privkey_import_url:
  * @pkey: The structure to store the parsed key
@@ -589,6 +638,7 @@ gnutls_pkcs11_privkey_generate2 (const char* url, gnutls_pk_algorithm_t pk,
   struct ck_mechanism mech;
   gnutls_pubkey_t pkey = NULL;
   gnutls_pkcs11_obj_t obj = NULL;
+  ck_key_type_t key_type;
 
   memset(&sinfo, 0, sizeof(sinfo));
 
@@ -615,7 +665,7 @@ gnutls_pkcs11_privkey_generate2 (const char* url, gnutls_pk_algorithm_t pk,
   a_val = p_val = 0;
   mech.parameter = NULL;
   mech.parameter_len = 0;
-  mech.mechanism = pk_to_genmech(pk);
+  mech.mechanism = pk_to_genmech(pk, &key_type);
 
   switch(pk)
     {
@@ -761,7 +811,7 @@ gnutls_pkcs11_privkey_generate2 (const char* url, gnutls_pk_algorithm_t pk,
 
       obj->pk_algorithm = pk;
       obj->type = GNUTLS_PKCS11_OBJ_PUBKEY;
-      ret = pkcs11_read_pubkey(sinfo.module, sinfo.pks, pub, mech.mechanism, obj->pubkey);
+      ret = pkcs11_read_pubkey(sinfo.module, sinfo.pks, pub, key_type, obj->pubkey);
       if (ret < 0)
         {
           gnutls_assert ();
