@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2012 Free Software Foundation, Inc.
+ * Copyright (C) 2003-2014 Free Software Foundation, Inc.
  *
  * Author: Nikos Mavrogiannopoulos
  *
@@ -33,6 +33,10 @@
 #include <common.h>
 #include <c-ctype.h>
 
+static int
+data2hex (const void * data, size_t data_size,
+          void * _out, size_t * sizeof_out);
+
 struct oid_to_string
 {
   const char *oid;
@@ -61,6 +65,7 @@ static const struct oid_to_string _oid2str[] = {
   {"2.5.4.3", "CN", "PKIX1.DirectoryString", ASN1_ETYPE_INVALID},
   {"2.5.4.7", "L", "PKIX1.DirectoryString", ASN1_ETYPE_INVALID},
   {"2.5.4.8", "ST", "PKIX1.DirectoryString", ASN1_ETYPE_INVALID},
+  {"2.5.4.13", "description", "PKIX1.DirectoryString", ASN1_ETYPE_INVALID},
 
   {"2.5.4.5", "serialNumber", NULL, ASN1_ETYPE_PRINTABLE_STRING},
   {"2.5.4.20", "telephoneNumber", NULL, ASN1_ETYPE_PRINTABLE_STRING},
@@ -135,6 +140,56 @@ const char* _gnutls_ldap_string_to_oid (const char* str, unsigned str_len)
   return NULL;
 }
 
+/* Escapes a string following the rules from RFC4514.
+ */
+static int
+str_escape (const gnutls_datum_t* str, gnutls_datum_t * escaped)
+{
+  unsigned int j, i;
+  uint8_t *buffer = NULL;
+  int ret;
+
+  if (str == NULL)
+    return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
+  
+  /* the string will be at most twice the original */
+  buffer = gnutls_malloc(str->size*2+2);
+  if (buffer == NULL)
+    return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
+
+  for (i = j = 0; i < str->size; i++)
+    {
+      if (str->data[i] == 0)
+        {
+          /* this is handled earlier */
+          ret = gnutls_assert_val(GNUTLS_E_ASN1_DER_ERROR);
+          goto cleanup;
+        }
+
+      if (str->data[i] == ',' || str->data[i] == '+' || str->data[i] == '"'
+          || str->data[i] == '\\' || str->data[i] == '<' || str->data[i] == '>'
+          || str->data[i] == ';' || str->data[i] == 0)
+        buffer[j++] = '\\';
+      else if (i==0 && str->data[i] == '#')
+        buffer[j++] = '\\';
+      else if (i==0 && str->data[i] == ' ')
+        buffer[j++] = '\\';
+      else if (i==(str->size-1) && str->data[i] == ' ')
+        buffer[j++] = '\\';
+      
+      buffer[j++] = str->data[i];
+    }
+
+  /* null terminate the string */
+  buffer[j] = 0;
+  escaped->data = buffer;
+  escaped->size = j;
+
+  return 0;
+cleanup:
+  gnutls_free(buffer);
+  return ret;
+}
 
 /**
  * gnutls_x509_dn_oid_known:
@@ -187,7 +242,7 @@ gnutls_x509_dn_oid_name (const char *oid, unsigned int flags)
 
   do
     {
-      if (strcmp (_oid2str[i].oid, oid) == 0)
+      if (strcmp (_oid2str[i].oid, oid) == 0 && _oid2str[i].ldap_desc != NULL)
         return _oid2str[i].ldap_desc;
       i++;
     }
@@ -251,7 +306,7 @@ size_t size;
         return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
 
       size = out->size;
-      ret = _gnutls_x509_data2hex (input->data, input->size, out->data, &size);
+      ret = data2hex (input->data, input->size, out->data, &size);
       if (ret < 0)
         {
           gnutls_assert();
@@ -371,6 +426,7 @@ _gnutls_x509_dn_to_string (const char *oid, void *value,
 {
   const struct oid_to_string* oentry;
   int ret;
+  gnutls_datum_t tmp;
   size_t size;
 
   if (value == NULL || value_size <= 0)
@@ -382,13 +438,14 @@ _gnutls_x509_dn_to_string (const char *oid, void *value,
   oentry = get_oid_entry(oid);
   if (oentry == NULL)
     { /* unknown OID -> hex */
+ unknown_oid:
       str->size = value_size*2+2;
       str->data = gnutls_malloc(str->size);
       if (str->data == NULL)
         return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
 
       size = str->size;
-      ret = _gnutls_x509_data2hex (value, value_size, str->data, &size);
+      ret = data2hex (value, value_size, str->data, &size);
       if (ret < 0)
         {
           gnutls_assert();
@@ -401,34 +458,37 @@ _gnutls_x509_dn_to_string (const char *oid, void *value,
   
   if (oentry->asn_desc != NULL)
     { /* complex */
-      ret = decode_complex_string(oentry, value, value_size, str);
+      ret = decode_complex_string(oentry, value, value_size, &tmp);
       if (ret < 0)
-        return gnutls_assert_val(ret);
+        goto unknown_oid;
     }
   else
     {
       ret = _gnutls_x509_decode_string(oentry->etype, value, value_size,
-                                       str);
+                                       &tmp);
       if (ret < 0)
-        {
-          return gnutls_assert_val(ret);
-        }
+        goto unknown_oid;
     }
 
+  ret = str_escape(&tmp, str);
+  _gnutls_free_datum (&tmp);
+
+  if (ret < 0)
+    return gnutls_assert_val(ret);
+
   return 0;
 }
 
-
 /* Converts a data string to an LDAP rfc2253 hex string
  * something like '#01020304'
  */
-int
-_gnutls_x509_data2hex (const void * data, size_t data_size,
-                       void * _out, size_t * sizeof_out)
+static int
+data2hex (const void * data, size_t data_size,
+          void * _out, size_t * sizeof_out)
 {
   char *res;
   char escaped[MAX_STRING_LEN];
-  unsigned int size;
+  unsigned int size, res_size;
   char* out = _out;
 
   if (2 * data_size + 1 > MAX_STRING_LEN)
@@ -444,10 +504,11 @@ _gnutls_x509_data2hex (const void * data, size_t data_size,
       return GNUTLS_E_INTERNAL_ERROR;
     }
 
-  size = strlen (res) + 1;
+  res_size = strlen(res);
+  size = res_size + 1; /* +1 for the '#' */
   if (size + 1 > *sizeof_out)
     {
-      *sizeof_out = size;
+      *sizeof_out = size + 1;
       return GNUTLS_E_SHORT_MEMORY_BUFFER;
     }
   *sizeof_out = size;           /* -1 for the null +1 for the '#' */
@@ -455,8 +516,8 @@ _gnutls_x509_data2hex (const void * data, size_t data_size,
   if (out)
     {
       out[0] = '#';
-      out[1] = 0;
-      _gnutls_str_cat (out, *sizeof_out, res);
+      memcpy(&out[1], res, res_size);
+      out[size] = 0;
     }
 
   return 0;
@@ -667,6 +728,16 @@ gtime2generalTime (time_t gtime, char *str_time, size_t str_time_size)
 {
   size_t ret;
   struct tm _tm;
+	
+  if (gtime == (time_t)-1
+#if SIZEOF_LONG == 8
+                       || gtime >= 253402210800
+#endif
+     )
+    {
+       	snprintf(str_time, str_time_size, "99991231235959Z");
+       	return 0;
+    }
 
   if (!gmtime_r (&gtime, &_tm))
     {
@@ -909,7 +980,7 @@ _gnutls_x509_decode_string (unsigned int etype,
 {
   int ret;
   const uint8_t *str;
-  unsigned int str_size;
+  unsigned int str_size, len;
   gnutls_datum_t td;
   
   ret = asn1_decode_simple_der (etype, der, der_size, &str, &str_size);
@@ -943,7 +1014,12 @@ _gnutls_x509_decode_string (unsigned int etype,
   /* Refuse to deal with strings containing NULs. */
   if (etype != ASN1_ETYPE_OCTET_STRING)
     {
-      if (strlen ((void*)output->data) != (size_t)output->size)
+      if (output->data)
+        len = strlen ((void*)output->data);
+      else
+        len = 0;
+
+      if (len != (size_t)output->size)
         {
           _gnutls_free_datum(output);
           ret = gnutls_assert_val(GNUTLS_E_ASN1_DER_ERROR);
@@ -1551,7 +1627,7 @@ _gnutls_x509_get_signature (ASN1_TYPE src, const char *src_name,
                             gnutls_datum_t * signature)
 {
   int result, len;
-  unsigned int bits;
+  int bits;
 
   signature->data = NULL;
   signature->size = 0;
@@ -1569,7 +1645,7 @@ _gnutls_x509_get_signature (ASN1_TYPE src, const char *src_name,
     }
 
   bits = len;
-  if (bits % 8 != 0)
+  if (bits % 8 != 0 || bits < 8)
     {
       gnutls_assert ();
       result = GNUTLS_E_CERTIFICATE_ERROR;
@@ -1799,3 +1875,35 @@ cleanup:
   
   return ret;                          
 }
+
+int
+_gnutls_x509_get_raw_dn2 (ASN1_TYPE c2, gnutls_datum_t* raw,
+                              const char *whom, gnutls_datum_t * dn)
+{
+  int result, len1;
+  int start1, end1;
+  result =
+    asn1_der_decoding_startEnd (c2, raw->data, raw->size,
+                                whom, &start1, &end1);
+
+  if (result != ASN1_SUCCESS)
+    {
+      gnutls_assert ();
+      result = _gnutls_asn2err (result);
+      goto cleanup;
+    }
+
+  len1 = end1 - start1 + 1;
+
+  result = _gnutls_set_datum (dn, &raw->data[start1], len1);
+  if (result < 0)
+    {
+      gnutls_assert();
+      goto cleanup;
+    }
+
+  result = 0;
+
+cleanup:
+  return result;
+}
